import express from 'express';
import { HttpStatusCode, LiveVideoLatencyMode, UserRight, VideoPrivacy, VideoState } from '@peertube/peertube-models';
import { exists } from '../../../helpers/custom-validators/misc.js';
import { createReqFiles } from '../../../helpers/express-utils.js';
import { getFormattedObjects } from '../../../helpers/utils.js';
import { ASSETS_PATH, MIMETYPES } from '../../../initializers/constants.js';
import { getLocalVideoActivityPubUrl } from '../../../lib/activitypub/url.js';
import { federateVideoIfNeeded } from '../../../lib/activitypub/videos/index.js';
import { Hooks } from '../../../lib/plugins/hooks.js';
import { buildLocalVideoFromReq, buildVideoThumbnailsFromReq, setVideoTags } from '../../../lib/video.js';
import { videoLiveAddValidator, videoLiveFindReplaySessionValidator, videoLiveGetValidator, videoLiveListSessionsValidator, videoLiveUpdateValidator } from '../../../middlewares/validators/videos/video-live.js';
import { VideoLiveReplaySettingModel } from '../../../models/video/video-live-replay-setting.js';
import { VideoLiveSessionModel } from '../../../models/video/video-live-session.js';
import { VideoLiveModel } from '../../../models/video/video-live.js';
import { VideoPasswordModel } from '../../../models/video/video-password.js';
import { buildUUID, uuidToShort } from '@peertube/peertube-node-utils';
import { logger } from '../../../helpers/logger.js';
import { sequelizeTypescript } from '../../../initializers/database.js';
import { updateLocalVideoMiniatureFromExisting } from '../../../lib/thumbnail.js';
import { asyncMiddleware, asyncRetryTransactionMiddleware, authenticate, optionalAuthenticate } from '../../../middlewares/index.js';
import { VideoModel } from '../../../models/video/video.js';
const liveRouter = express.Router();
const reqVideoFileLive = createReqFiles(['thumbnailfile', 'previewfile'], MIMETYPES.IMAGE.MIMETYPE_EXT);
liveRouter.post('/live', authenticate, reqVideoFileLive, asyncMiddleware(videoLiveAddValidator), asyncRetryTransactionMiddleware(addLiveVideo));
liveRouter.get('/live/:videoId/sessions', authenticate, asyncMiddleware(videoLiveGetValidator), videoLiveListSessionsValidator, asyncMiddleware(getLiveVideoSessions));
liveRouter.get('/live/:videoId', optionalAuthenticate, asyncMiddleware(videoLiveGetValidator), getLiveVideo);
liveRouter.put('/live/:videoId', authenticate, asyncMiddleware(videoLiveGetValidator), videoLiveUpdateValidator, asyncRetryTransactionMiddleware(updateLiveVideo));
liveRouter.get('/:videoId/live-session', asyncMiddleware(videoLiveFindReplaySessionValidator), getLiveReplaySession);
export { liveRouter };
function getLiveVideo(req, res) {
    const videoLive = res.locals.videoLive;
    return res.json(videoLive.toFormattedJSON(canSeePrivateLiveInformation(res)));
}
function getLiveReplaySession(req, res) {
    const session = res.locals.videoLiveSession;
    return res.json(session.toFormattedJSON());
}
async function getLiveVideoSessions(req, res) {
    const videoLive = res.locals.videoLive;
    const data = await VideoLiveSessionModel.listSessionsOfLiveForAPI({ videoId: videoLive.videoId });
    return res.json(getFormattedObjects(data, data.length));
}
function canSeePrivateLiveInformation(res) {
    var _a;
    const user = (_a = res.locals.oauth) === null || _a === void 0 ? void 0 : _a.token.User;
    if (!user)
        return false;
    if (user.hasRight(UserRight.GET_ANY_LIVE))
        return true;
    const video = res.locals.videoAll;
    return video.VideoChannel.Account.userId === user.id;
}
async function updateLiveVideo(req, res) {
    const body = req.body;
    const video = res.locals.videoAll;
    const videoLive = res.locals.videoLive;
    const newReplaySettingModel = await updateReplaySettings(videoLive, body);
    if (newReplaySettingModel)
        videoLive.replaySettingId = newReplaySettingModel.id;
    else
        videoLive.replaySettingId = null;
    if (exists(body.permanentLive))
        videoLive.permanentLive = body.permanentLive;
    if (exists(body.latencyMode))
        videoLive.latencyMode = body.latencyMode;
    video.VideoLive = await videoLive.save();
    await federateVideoIfNeeded(video, false);
    return res.status(HttpStatusCode.NO_CONTENT_204).end();
}
async function updateReplaySettings(videoLive, body) {
    if (exists(body.saveReplay))
        videoLive.saveReplay = body.saveReplay;
    if (!videoLive.saveReplay) {
        if (videoLive.replaySettingId) {
            await VideoLiveReplaySettingModel.removeSettings(videoLive.replaySettingId);
        }
        return undefined;
    }
    const settingModel = videoLive.replaySettingId
        ? await VideoLiveReplaySettingModel.load(videoLive.replaySettingId)
        : new VideoLiveReplaySettingModel();
    if (exists(body.replaySettings.privacy))
        settingModel.privacy = body.replaySettings.privacy;
    return settingModel.save();
}
async function addLiveVideo(req, res) {
    const videoInfo = req.body;
    let videoData = buildLocalVideoFromReq(videoInfo, res.locals.videoChannel.id);
    videoData = await Hooks.wrapObject(videoData, 'filter:api.video.live.video-attribute.result');
    videoData.isLive = true;
    videoData.state = VideoState.WAITING_FOR_LIVE;
    videoData.duration = 0;
    const video = new VideoModel(videoData);
    video.url = getLocalVideoActivityPubUrl(video);
    const videoLive = new VideoLiveModel();
    videoLive.saveReplay = videoInfo.saveReplay || false;
    videoLive.permanentLive = videoInfo.permanentLive || false;
    videoLive.latencyMode = videoInfo.latencyMode || LiveVideoLatencyMode.DEFAULT;
    videoLive.streamKey = buildUUID();
    const [thumbnailModel, previewModel] = await buildVideoThumbnailsFromReq({
        video,
        files: req.files,
        fallback: type => {
            return updateLocalVideoMiniatureFromExisting({
                inputPath: ASSETS_PATH.DEFAULT_LIVE_BACKGROUND,
                video,
                type,
                automaticallyGenerated: true,
                keepOriginal: true
            });
        }
    });
    const { videoCreated } = await sequelizeTypescript.transaction(async (t) => {
        var _a, _b;
        const sequelizeOptions = { transaction: t };
        const videoCreated = await video.save(sequelizeOptions);
        if (thumbnailModel)
            await videoCreated.addAndSaveThumbnail(thumbnailModel, t);
        if (previewModel)
            await videoCreated.addAndSaveThumbnail(previewModel, t);
        videoCreated.VideoChannel = res.locals.videoChannel;
        if (videoLive.saveReplay) {
            const replaySettings = new VideoLiveReplaySettingModel({
                privacy: (_b = (_a = videoInfo.replaySettings) === null || _a === void 0 ? void 0 : _a.privacy) !== null && _b !== void 0 ? _b : videoCreated.privacy
            });
            await replaySettings.save(sequelizeOptions);
            videoLive.replaySettingId = replaySettings.id;
        }
        videoLive.videoId = videoCreated.id;
        videoCreated.VideoLive = await videoLive.save(sequelizeOptions);
        await setVideoTags({ video, tags: videoInfo.tags, transaction: t });
        await federateVideoIfNeeded(videoCreated, true, t);
        if (videoInfo.privacy === VideoPrivacy.PASSWORD_PROTECTED) {
            await VideoPasswordModel.addPasswords(videoInfo.videoPasswords, video.id, t);
        }
        logger.info('Video live %s with uuid %s created.', videoInfo.name, videoCreated.uuid);
        return { videoCreated };
    });
    Hooks.runAction('action:api.live-video.created', { video: videoCreated, req, res });
    return res.json({
        video: {
            id: videoCreated.id,
            shortUUID: uuidToShort(videoCreated.uuid),
            uuid: videoCreated.uuid
        }
    });
}
//# sourceMappingURL=live.js.map