import express from 'express';
import { move } from 'fs-extra/esm';
import { basename } from 'path';
import { getResumableUploadPath } from '../../../helpers/upload.js';
import { getLocalVideoActivityPubUrl } from '../../../lib/activitypub/url.js';
import { JobQueue } from '../../../lib/job-queue/index.js';
import { Redis } from '../../../lib/redis.js';
import { uploadx } from '../../../lib/uploadx.js';
import { buildLocalVideoFromReq, buildMoveJob, buildVideoThumbnailsFromReq, setVideoTags } from '../../../lib/video.js';
import { buildNewFile } from '../../../lib/video-file.js';
import { VideoPathManager } from '../../../lib/video-path-manager.js';
import { buildNextVideoState } from '../../../lib/video-state.js';
import { openapiOperationDoc } from '../../../middlewares/doc.js';
import { VideoPasswordModel } from '../../../models/video/video-password.js';
import { VideoSourceModel } from '../../../models/video/video-source.js';
import { uuidToShort } from '@peertube/peertube-node-utils';
import { HttpStatusCode, ThumbnailType, VideoPrivacy, VideoState } from '@peertube/peertube-models';
import { auditLoggerFactory, getAuditIdFromRes, VideoAuditView } from '../../../helpers/audit-logger.js';
import { createReqFiles } from '../../../helpers/express-utils.js';
import { logger, loggerTagsFactory } from '../../../helpers/logger.js';
import { CONSTRAINTS_FIELDS, MIMETYPES } from '../../../initializers/constants.js';
import { sequelizeTypescript } from '../../../initializers/database.js';
import { Hooks } from '../../../lib/plugins/hooks.js';
import { generateLocalVideoMiniature } from '../../../lib/thumbnail.js';
import { autoBlacklistVideoIfNeeded } from '../../../lib/video-blacklist.js';
import { asyncMiddleware, asyncRetryTransactionMiddleware, authenticate, videosAddLegacyValidator, videosAddResumableInitValidator, videosAddResumableValidator } from '../../../middlewares/index.js';
import { ScheduleVideoUpdateModel } from '../../../models/video/schedule-video-update.js';
import { VideoModel } from '../../../models/video/video.js';
import { ffprobePromise, getChaptersFromContainer } from '@peertube/peertube-ffmpeg';
import { replaceChapters, replaceChaptersFromDescriptionIfNeeded } from '../../../lib/video-chapters.js';
const lTags = loggerTagsFactory('api', 'video');
const auditLogger = auditLoggerFactory('videos');
const uploadRouter = express.Router();
const reqVideoFileAdd = createReqFiles(['videofile', 'thumbnailfile', 'previewfile'], Object.assign(Object.assign({}, MIMETYPES.VIDEO.MIMETYPE_EXT), MIMETYPES.IMAGE.MIMETYPE_EXT));
const reqVideoFileAddResumable = createReqFiles(['thumbnailfile', 'previewfile'], MIMETYPES.IMAGE.MIMETYPE_EXT, getResumableUploadPath());
uploadRouter.post('/upload', openapiOperationDoc({ operationId: 'uploadLegacy' }), authenticate, reqVideoFileAdd, asyncMiddleware(videosAddLegacyValidator), asyncRetryTransactionMiddleware(addVideoLegacy));
uploadRouter.post('/upload-resumable', openapiOperationDoc({ operationId: 'uploadResumableInit' }), authenticate, reqVideoFileAddResumable, asyncMiddleware(videosAddResumableInitValidator), (req, res) => uploadx.upload(req, res));
uploadRouter.delete('/upload-resumable', authenticate, asyncMiddleware(deleteUploadResumableCache), (req, res) => uploadx.upload(req, res));
uploadRouter.put('/upload-resumable', openapiOperationDoc({ operationId: 'uploadResumable' }), authenticate, uploadx.upload, asyncMiddleware(videosAddResumableValidator), asyncMiddleware(addVideoResumable));
export { uploadRouter };
async function addVideoLegacy(req, res) {
    req.setTimeout(1000 * 60 * 10, () => {
        logger.error('Video upload has timed out.');
        return res.fail({
            status: HttpStatusCode.REQUEST_TIMEOUT_408,
            message: 'Video upload has timed out.'
        });
    });
    const videoPhysicalFile = req.files['videofile'][0];
    const videoInfo = req.body;
    const files = req.files;
    const response = await addVideo({ req, res, videoPhysicalFile, videoInfo, files });
    return res.json(response);
}
async function addVideoResumable(req, res) {
    const videoPhysicalFile = res.locals.uploadVideoFileResumable;
    const videoInfo = videoPhysicalFile.metadata;
    const files = { previewfile: videoInfo.previewfile, thumbnailfile: videoInfo.thumbnailfile };
    const response = await addVideo({ req, res, videoPhysicalFile, videoInfo, files });
    await Redis.Instance.setUploadSession(req.query.upload_id, response);
    return res.json(response);
}
async function addVideo(options) {
    const { req, res, videoPhysicalFile, videoInfo, files } = options;
    const videoChannel = res.locals.videoChannel;
    const user = res.locals.oauth.token.User;
    let videoData = buildLocalVideoFromReq(videoInfo, videoChannel.id);
    videoData = await Hooks.wrapObject(videoData, 'filter:api.video.upload.video-attribute.result');
    videoData.state = buildNextVideoState();
    videoData.duration = videoPhysicalFile.duration;
    const video = new VideoModel(videoData);
    video.VideoChannel = videoChannel;
    video.url = getLocalVideoActivityPubUrl(video);
    const ffprobe = await ffprobePromise(videoPhysicalFile.path);
    const videoFile = await buildNewFile({ path: videoPhysicalFile.path, mode: 'web-video', ffprobe });
    const originalFilename = videoPhysicalFile.originalname;
    const containerChapters = await getChaptersFromContainer({
        path: videoPhysicalFile.path,
        maxTitleLength: CONSTRAINTS_FIELDS.VIDEO_CHAPTERS.TITLE.max,
        ffprobe
    });
    logger.debug(`Got ${containerChapters.length} chapters from video "${video.name}" container`, Object.assign({ containerChapters }, lTags(video.uuid)));
    const destination = VideoPathManager.Instance.getFSVideoFileOutputPath(video, videoFile);
    await move(videoPhysicalFile.path, destination);
    videoPhysicalFile.filename = basename(destination);
    videoPhysicalFile.path = destination;
    const thumbnails = await createThumbnailFiles({ video, files, videoFile, ffprobe });
    const { videoCreated } = await sequelizeTypescript.transaction(async (t) => {
        const sequelizeOptions = { transaction: t };
        const videoCreated = await video.save(sequelizeOptions);
        for (const thumbnail of thumbnails) {
            await videoCreated.addAndSaveThumbnail(thumbnail, t);
        }
        videoCreated.VideoChannel = res.locals.videoChannel;
        videoFile.videoId = video.id;
        await videoFile.save(sequelizeOptions);
        video.VideoFiles = [videoFile];
        await VideoSourceModel.create({
            filename: originalFilename,
            videoId: video.id
        }, { transaction: t });
        await setVideoTags({ video, tags: videoInfo.tags, transaction: t });
        if (videoInfo.scheduleUpdate) {
            await ScheduleVideoUpdateModel.create({
                videoId: video.id,
                updateAt: new Date(videoInfo.scheduleUpdate.updateAt),
                privacy: videoInfo.scheduleUpdate.privacy || null
            }, sequelizeOptions);
        }
        if (!await replaceChaptersFromDescriptionIfNeeded({ newDescription: video.description, video, transaction: t })) {
            await replaceChapters({ video, chapters: containerChapters, transaction: t });
        }
        await autoBlacklistVideoIfNeeded({
            video,
            user,
            isRemote: false,
            isNew: true,
            isNewFile: true,
            transaction: t
        });
        if (videoInfo.privacy === VideoPrivacy.PASSWORD_PROTECTED) {
            await VideoPasswordModel.addPasswords(videoInfo.videoPasswords, video.id, t);
        }
        auditLogger.create(getAuditIdFromRes(res), new VideoAuditView(videoCreated.toFormattedDetailsJSON()));
        logger.info('Video with name %s and uuid %s created.', videoInfo.name, videoCreated.uuid, lTags(videoCreated.uuid));
        return { videoCreated };
    });
    await videoCreated.VideoChannel.setAsUpdated();
    addVideoJobsAfterUpload(videoCreated, videoFile)
        .catch(err => logger.error('Cannot build new video jobs of %s.', videoCreated.uuid, Object.assign({ err }, lTags(videoCreated.uuid))));
    Hooks.runAction('action:api.video.uploaded', { video: videoCreated, req, res });
    return {
        video: {
            id: videoCreated.id,
            shortUUID: uuidToShort(videoCreated.uuid),
            uuid: videoCreated.uuid
        }
    };
}
async function addVideoJobsAfterUpload(video, videoFile) {
    const jobs = [
        {
            type: 'manage-video-torrent',
            payload: {
                videoId: video.id,
                videoFileId: videoFile.id,
                action: 'create'
            }
        },
        {
            type: 'generate-video-storyboard',
            payload: {
                videoUUID: video.uuid,
                federate: false
            }
        },
        {
            type: 'notify',
            payload: {
                action: 'new-video',
                videoUUID: video.uuid
            }
        },
        {
            type: 'federate-video',
            payload: {
                videoUUID: video.uuid,
                isNewVideoForFederation: true
            }
        }
    ];
    if (video.state === VideoState.TO_MOVE_TO_EXTERNAL_STORAGE) {
        jobs.push(await buildMoveJob({ video, previousVideoState: undefined, type: 'move-to-object-storage' }));
    }
    if (video.state === VideoState.TO_TRANSCODE) {
        jobs.push({
            type: 'transcoding-job-builder',
            payload: {
                videoUUID: video.uuid,
                optimizeJob: {
                    isNewVideo: true
                }
            }
        });
    }
    return JobQueue.Instance.createSequentialJobFlow(...jobs);
}
async function deleteUploadResumableCache(req, res, next) {
    await Redis.Instance.deleteUploadSession(req.query.upload_id);
    return next();
}
async function createThumbnailFiles(options) {
    const { video, videoFile, files, ffprobe } = options;
    const models = await buildVideoThumbnailsFromReq({
        video,
        files,
        fallback: () => Promise.resolve(undefined)
    });
    const filteredModels = models.filter(m => !!m);
    const thumbnailsToGenerate = [ThumbnailType.MINIATURE, ThumbnailType.PREVIEW].filter(type => {
        return !filteredModels.some(m => m.type === type);
    });
    return [...filteredModels, ...await generateLocalVideoMiniature({ video, videoFile, types: thumbnailsToGenerate, ffprobe })];
}
//# sourceMappingURL=upload.js.map