import { basename, join } from 'path';
import { logger } from '../../helpers/logger.js';
import { CONFIG } from '../../initializers/config.js';
import { getHLSDirectory } from '../paths.js';
import { VideoPathManager } from '../video-path-manager.js';
import { generateHLSObjectBaseStorageKey, generateHLSObjectStorageKey, generateWebVideoObjectStorageKey } from './keys.js';
import { createObjectReadStream, listKeysOfPrefix, lTags, makeAvailable, removeObject, removeObjectByFullKey, removePrefix, storeContent, storeObject, updateObjectACL, updatePrefixACL } from './shared/index.js';
function listHLSFileKeysOf(playlist) {
    return listKeysOfPrefix(generateHLSObjectBaseStorageKey(playlist), CONFIG.OBJECT_STORAGE.STREAMING_PLAYLISTS);
}
function storeHLSFileFromFilename(playlist, filename) {
    return storeObject({
        inputPath: join(getHLSDirectory(playlist.Video), filename),
        objectStorageKey: generateHLSObjectStorageKey(playlist, filename),
        bucketInfo: CONFIG.OBJECT_STORAGE.STREAMING_PLAYLISTS,
        isPrivate: playlist.Video.hasPrivateStaticPath()
    });
}
function storeHLSFileFromPath(playlist, path) {
    return storeObject({
        inputPath: path,
        objectStorageKey: generateHLSObjectStorageKey(playlist, basename(path)),
        bucketInfo: CONFIG.OBJECT_STORAGE.STREAMING_PLAYLISTS,
        isPrivate: playlist.Video.hasPrivateStaticPath()
    });
}
function storeHLSFileFromContent(playlist, path, content) {
    return storeContent({
        content,
        inputPath: path,
        objectStorageKey: generateHLSObjectStorageKey(playlist, basename(path)),
        bucketInfo: CONFIG.OBJECT_STORAGE.STREAMING_PLAYLISTS,
        isPrivate: playlist.Video.hasPrivateStaticPath()
    });
}
function storeWebVideoFile(video, file) {
    return storeObject({
        inputPath: VideoPathManager.Instance.getFSVideoFileOutputPath(video, file),
        objectStorageKey: generateWebVideoObjectStorageKey(file.filename),
        bucketInfo: CONFIG.OBJECT_STORAGE.WEB_VIDEOS,
        isPrivate: video.hasPrivateStaticPath()
    });
}
async function updateWebVideoFileACL(video, file) {
    await updateObjectACL({
        objectStorageKey: generateWebVideoObjectStorageKey(file.filename),
        bucketInfo: CONFIG.OBJECT_STORAGE.WEB_VIDEOS,
        isPrivate: video.hasPrivateStaticPath()
    });
}
async function updateHLSFilesACL(playlist) {
    await updatePrefixACL({
        prefix: generateHLSObjectBaseStorageKey(playlist),
        bucketInfo: CONFIG.OBJECT_STORAGE.STREAMING_PLAYLISTS,
        isPrivate: playlist.Video.hasPrivateStaticPath()
    });
}
function removeHLSObjectStorage(playlist) {
    return removePrefix(generateHLSObjectBaseStorageKey(playlist), CONFIG.OBJECT_STORAGE.STREAMING_PLAYLISTS);
}
function removeHLSFileObjectStorageByFilename(playlist, filename) {
    return removeObject(generateHLSObjectStorageKey(playlist, filename), CONFIG.OBJECT_STORAGE.STREAMING_PLAYLISTS);
}
function removeHLSFileObjectStorageByPath(playlist, path) {
    return removeObject(generateHLSObjectStorageKey(playlist, basename(path)), CONFIG.OBJECT_STORAGE.STREAMING_PLAYLISTS);
}
function removeHLSFileObjectStorageByFullKey(key) {
    return removeObjectByFullKey(key, CONFIG.OBJECT_STORAGE.STREAMING_PLAYLISTS);
}
function removeWebVideoObjectStorage(videoFile) {
    return removeObject(generateWebVideoObjectStorageKey(videoFile.filename), CONFIG.OBJECT_STORAGE.WEB_VIDEOS);
}
async function makeHLSFileAvailable(playlist, filename, destination) {
    const key = generateHLSObjectStorageKey(playlist, filename);
    logger.info('Fetching HLS file %s from object storage to %s.', key, destination, lTags());
    await makeAvailable({
        key,
        destination,
        bucketInfo: CONFIG.OBJECT_STORAGE.STREAMING_PLAYLISTS
    });
    return destination;
}
async function makeWebVideoFileAvailable(filename, destination) {
    const key = generateWebVideoObjectStorageKey(filename);
    logger.info('Fetching Web Video file %s from object storage to %s.', key, destination, lTags());
    await makeAvailable({
        key,
        destination,
        bucketInfo: CONFIG.OBJECT_STORAGE.WEB_VIDEOS
    });
    return destination;
}
function getWebVideoFileReadStream(options) {
    const { filename, rangeHeader } = options;
    const key = generateWebVideoObjectStorageKey(filename);
    return createObjectReadStream({
        key,
        bucketInfo: CONFIG.OBJECT_STORAGE.WEB_VIDEOS,
        rangeHeader
    });
}
function getHLSFileReadStream(options) {
    const { playlist, filename, rangeHeader } = options;
    const key = generateHLSObjectStorageKey(playlist, filename);
    return createObjectReadStream({
        key,
        bucketInfo: CONFIG.OBJECT_STORAGE.STREAMING_PLAYLISTS,
        rangeHeader
    });
}
export { listHLSFileKeysOf, storeWebVideoFile, storeHLSFileFromFilename, storeHLSFileFromPath, storeHLSFileFromContent, updateWebVideoFileACL, updateHLSFilesACL, removeHLSObjectStorage, removeHLSFileObjectStorageByFilename, removeHLSFileObjectStorageByPath, removeHLSFileObjectStorageByFullKey, removeWebVideoObjectStorage, makeWebVideoFileAvailable, makeHLSFileAvailable, getWebVideoFileReadStream, getHLSFileReadStream };
//# sourceMappingURL=videos.js.map