import { move } from 'fs-extra/esm';
import { dirname, join } from 'path';
import { logger } from '../../../helpers/logger.js';
import { renameVideoFileInPlaylist } from '../../hls.js';
import { getHlsResolutionPlaylistFilename } from '../../paths.js';
import { onTranscodingEnded } from '../../transcoding/ended-transcoding.js';
import { onHLSVideoFileTranscoding } from '../../transcoding/hls-transcoding.js';
import { buildNewFile, removeAllWebVideoFiles } from '../../video-file.js';
import { VideoJobInfoModel } from '../../../models/video/video-job-info.js';
import { pick } from '@peertube/peertube-core-utils';
import { buildUUID } from '@peertube/peertube-node-utils';
import { generateRunnerTranscodingVideoInputFileUrl } from '../runner-urls.js';
import { AbstractVODTranscodingJobHandler } from './abstract-vod-transcoding-job-handler.js';
import { loadTranscodingRunnerVideo } from './shared/index.js';
export class VODHLSTranscodingJobHandler extends AbstractVODTranscodingJobHandler {
    async create(options) {
        const { video, resolution, fps, dependsOnRunnerJob, priority } = options;
        const jobUUID = buildUUID();
        const payload = {
            input: {
                videoFileUrl: generateRunnerTranscodingVideoInputFileUrl(jobUUID, video.uuid)
            },
            output: {
                resolution,
                fps
            }
        };
        const privatePayload = Object.assign(Object.assign({}, pick(options, ['isNewVideo', 'deleteWebVideoFiles'])), { videoUUID: video.uuid });
        const job = await this.createRunnerJob({
            type: 'vod-hls-transcoding',
            jobUUID,
            payload,
            privatePayload,
            priority,
            dependsOnRunnerJob
        });
        await VideoJobInfoModel.increaseOrCreate(video.uuid, 'pendingTranscode');
        return job;
    }
    async specificComplete(options) {
        const { runnerJob, resultPayload } = options;
        const privatePayload = runnerJob.privatePayload;
        const video = await loadTranscodingRunnerVideo(runnerJob, this.lTags);
        if (!video)
            return;
        const videoFilePath = resultPayload.videoFile;
        const resolutionPlaylistFilePath = resultPayload.resolutionPlaylistFile;
        const videoFile = await buildNewFile({ path: videoFilePath, mode: 'hls' });
        const newVideoFilePath = join(dirname(videoFilePath), videoFile.filename);
        await move(videoFilePath, newVideoFilePath);
        const resolutionPlaylistFilename = getHlsResolutionPlaylistFilename(videoFile.filename);
        const newResolutionPlaylistFilePath = join(dirname(resolutionPlaylistFilePath), resolutionPlaylistFilename);
        await move(resolutionPlaylistFilePath, newResolutionPlaylistFilePath);
        await renameVideoFileInPlaylist(newResolutionPlaylistFilePath, videoFile.filename);
        await onHLSVideoFileTranscoding({
            video,
            videoFile,
            m3u8OutputPath: newResolutionPlaylistFilePath,
            videoOutputPath: newVideoFilePath
        });
        await onTranscodingEnded({ isNewVideo: privatePayload.isNewVideo, moveVideoToNextState: true, video });
        if (privatePayload.deleteWebVideoFiles === true) {
            logger.info('Removing web video files of %s now we have a HLS version of it.', video.uuid, this.lTags(video.uuid));
            await removeAllWebVideoFiles(video);
        }
        logger.info('Runner VOD HLS job %s for %s ended.', runnerJob.uuid, video.uuid, this.lTags(runnerJob.uuid, video.uuid));
    }
}
//# sourceMappingURL=vod-hls-transcoding-job-handler.js.map