import memoizee from 'memoizee';
import { ThumbnailType, VideoPrivacy } from '@peertube/peertube-models';
import { CONFIG } from '../initializers/config.js';
import { MEMOIZE_LENGTH, MEMOIZE_TTL } from '../initializers/constants.js';
import { TagModel } from '../models/video/tag.js';
import { VideoJobInfoModel } from '../models/video/video-job-info.js';
import { VideoModel } from '../models/video/video.js';
import { JobQueue } from './job-queue/job-queue.js';
import { updateLocalVideoMiniatureFromExisting } from './thumbnail.js';
import { moveFilesIfPrivacyChanged } from './video-privacy.js';
export function buildLocalVideoFromReq(videoInfo, channelId) {
    var _a, _b, _c;
    return {
        name: videoInfo.name,
        remote: false,
        category: videoInfo.category,
        licence: (_a = videoInfo.licence) !== null && _a !== void 0 ? _a : CONFIG.DEFAULTS.PUBLISH.LICENCE,
        language: videoInfo.language,
        commentsEnabled: (_b = videoInfo.commentsEnabled) !== null && _b !== void 0 ? _b : CONFIG.DEFAULTS.PUBLISH.COMMENTS_ENABLED,
        downloadEnabled: (_c = videoInfo.downloadEnabled) !== null && _c !== void 0 ? _c : CONFIG.DEFAULTS.PUBLISH.DOWNLOAD_ENABLED,
        waitTranscoding: videoInfo.waitTranscoding || false,
        nsfw: videoInfo.nsfw || false,
        description: videoInfo.description,
        support: videoInfo.support,
        privacy: videoInfo.privacy || VideoPrivacy.PRIVATE,
        channelId,
        originallyPublishedAt: videoInfo.originallyPublishedAt
            ? new Date(videoInfo.originallyPublishedAt)
            : null
    };
}
export async function buildVideoThumbnailsFromReq(options) {
    const { video, files, fallback, automaticallyGenerated } = options;
    const promises = [
        {
            type: ThumbnailType.MINIATURE,
            fieldName: 'thumbnailfile'
        },
        {
            type: ThumbnailType.PREVIEW,
            fieldName: 'previewfile'
        }
    ].map(p => {
        const fields = files === null || files === void 0 ? void 0 : files[p.fieldName];
        if (fields) {
            return updateLocalVideoMiniatureFromExisting({
                inputPath: fields[0].path,
                video,
                type: p.type,
                automaticallyGenerated: automaticallyGenerated || false
            });
        }
        return fallback(p.type);
    });
    return Promise.all(promises);
}
export async function setVideoTags(options) {
    const { video, tags, transaction } = options;
    const internalTags = tags || [];
    const tagInstances = await TagModel.findOrCreateTags(internalTags, transaction);
    await video.$set('Tags', tagInstances, { transaction });
    video.Tags = tagInstances;
}
export async function buildMoveJob(options) {
    const { video, previousVideoState, isNewVideo = true, type } = options;
    await VideoJobInfoModel.increaseOrCreate(video.uuid, 'pendingMove');
    return {
        type,
        payload: {
            videoUUID: video.uuid,
            isNewVideo,
            previousVideoState
        }
    };
}
export async function getVideoDuration(videoId) {
    const video = await VideoModel.load(videoId);
    const duration = video.isLive
        ? undefined
        : video.duration;
    return { duration, isLive: video.isLive };
}
export const getCachedVideoDuration = memoizee(getVideoDuration, {
    promise: true,
    max: MEMOIZE_LENGTH.VIDEO_DURATION,
    maxAge: MEMOIZE_TTL.VIDEO_DURATION
});
export async function addVideoJobsAfterUpdate(options) {
    const { video, nameChanged, oldPrivacy, isNewVideoForFederation } = options;
    const jobs = [];
    const filePathChanged = await moveFilesIfPrivacyChanged(video, oldPrivacy);
    if (!video.isLive && (nameChanged || filePathChanged)) {
        for (const file of (video.VideoFiles || [])) {
            const payload = { action: 'update-metadata', videoId: video.id, videoFileId: file.id };
            jobs.push({ type: 'manage-video-torrent', payload });
        }
        const hls = video.getHLSPlaylist();
        for (const file of ((hls === null || hls === void 0 ? void 0 : hls.VideoFiles) || [])) {
            const payload = { action: 'update-metadata', streamingPlaylistId: hls.id, videoFileId: file.id };
            jobs.push({ type: 'manage-video-torrent', payload });
        }
    }
    jobs.push({
        type: 'federate-video',
        payload: {
            videoUUID: video.uuid,
            isNewVideoForFederation
        }
    });
    const wasConfidentialVideo = new Set([
        VideoPrivacy.PRIVATE,
        VideoPrivacy.UNLISTED,
        VideoPrivacy.INTERNAL
    ]).has(oldPrivacy);
    if (wasConfidentialVideo) {
        jobs.push({
            type: 'notify',
            payload: {
                action: 'new-video',
                videoUUID: video.uuid
            }
        });
    }
    return JobQueue.Instance.createSequentialJobFlow(...jobs);
}
//# sourceMappingURL=video.js.map