import { buildUUID, isTestOrDevInstance, sha256 } from '@peertube/peertube-node-utils';
import { logger, loggerTagsFactory } from '../../../helpers/logger.js';
import { VIEW_LIFETIME } from '../../../initializers/constants.js';
import { sendView } from '../../activitypub/send/send-view.js';
import { PeerTubeSocket } from '../../peertube-socket.js';
import { getServerActor } from '../../../models/application/application.js';
import { VideoModel } from '../../../models/video/video.js';
const lTags = loggerTagsFactory('views');
export class VideoViewerCounters {
    constructor() {
        this.viewersPerVideo = new Map();
        this.idToViewer = new Map();
        this.salt = buildUUID();
        this.processingViewerCounters = false;
        setInterval(() => this.cleanViewerCounters(), VIEW_LIFETIME.VIEWER_COUNTER);
    }
    async addLocalViewer(options) {
        const { video, ip } = options;
        logger.debug('Adding local viewer to video viewers counter %s.', video.uuid, Object.assign({}, lTags(video.uuid)));
        const viewerId = this.generateViewerId(ip, video.uuid);
        const viewer = this.idToViewer.get(viewerId);
        if (viewer) {
            viewer.expires = this.buildViewerExpireTime();
            await this.federateViewerIfNeeded(video, viewer);
            return false;
        }
        const newViewer = await this.addViewerToVideo({ viewerId, video, viewerScope: 'local' });
        await this.federateViewerIfNeeded(video, newViewer);
        return true;
    }
    async addRemoteViewer(options) {
        const { video, viewerExpires, viewerId } = options;
        logger.debug('Adding remote viewer to video %s.', video.uuid, Object.assign({}, lTags(video.uuid)));
        await this.addViewerToVideo({ video, viewerExpires, viewerId, viewerScope: 'remote' });
        return true;
    }
    getTotalViewers(options) {
        let total = 0;
        for (const viewers of this.viewersPerVideo.values()) {
            total += viewers.filter(v => v.viewerScope === options.viewerScope && v.videoScope === options.videoScope).length;
        }
        return total;
    }
    getViewers(video) {
        const viewers = this.viewersPerVideo.get(video.id);
        if (!viewers)
            return 0;
        return viewers.length;
    }
    buildViewerExpireTime() {
        return new Date().getTime() + VIEW_LIFETIME.VIEWER_COUNTER;
    }
    async addViewerToVideo(options) {
        const { video, viewerExpires, viewerId, viewerScope } = options;
        let watchers = this.viewersPerVideo.get(video.id);
        if (!watchers) {
            watchers = [];
            this.viewersPerVideo.set(video.id, watchers);
        }
        const expires = viewerExpires
            ? viewerExpires.getTime()
            : this.buildViewerExpireTime();
        const videoScope = video.remote
            ? 'remote'
            : 'local';
        const viewer = { id: viewerId, expires, videoScope, viewerScope };
        watchers.push(viewer);
        this.idToViewer.set(viewerId, viewer);
        await this.notifyClients(video.id, watchers.length);
        return viewer;
    }
    async cleanViewerCounters() {
        if (this.processingViewerCounters)
            return;
        this.processingViewerCounters = true;
        if (!isTestOrDevInstance())
            logger.info('Cleaning video viewers.', lTags());
        try {
            for (const videoId of this.viewersPerVideo.keys()) {
                const notBefore = new Date().getTime();
                const viewers = this.viewersPerVideo.get(videoId);
                const newViewers = [];
                for (const viewer of viewers) {
                    if (viewer.expires > notBefore) {
                        newViewers.push(viewer);
                    }
                    else {
                        this.idToViewer.delete(viewer.id);
                    }
                }
                if (newViewers.length === 0)
                    this.viewersPerVideo.delete(videoId);
                else
                    this.viewersPerVideo.set(videoId, newViewers);
                await this.notifyClients(videoId, newViewers.length);
            }
        }
        catch (err) {
            logger.error('Error in video clean viewers scheduler.', Object.assign({ err }, lTags()));
        }
        this.processingViewerCounters = false;
    }
    async notifyClients(videoId, viewersLength) {
        const video = await VideoModel.loadImmutableAttributes(videoId);
        if (!video)
            return;
        PeerTubeSocket.Instance.sendVideoViewsUpdate(video, viewersLength);
        logger.debug('Video viewers update for %s is %d.', video.url, viewersLength, lTags());
    }
    generateViewerId(ip, videoUUID) {
        return sha256(this.salt + '-' + ip + '-' + videoUUID);
    }
    async federateViewerIfNeeded(video, viewer) {
        const now = new Date().getTime();
        const federationLimit = now - (VIEW_LIFETIME.VIEWER_COUNTER * 0.75);
        if (viewer.lastFederation && viewer.lastFederation > federationLimit)
            return;
        await sendView({ byActor: await getServerActor(), video, type: 'viewer', viewerIdentifier: viewer.id });
        viewer.lastFederation = now;
    }
}
//# sourceMappingURL=video-viewer-counters.js.map