import { logger, loggerTagsFactory } from '../../../helpers/logger.js';
import { sendView } from '../../activitypub/send/send-view.js';
import { getCachedVideoDuration } from '../../video.js';
import { getServerActor } from '../../../models/application/application.js';
import { buildUUID } from '@peertube/peertube-node-utils';
import { Redis } from '../../redis.js';
import { LRUCache } from 'lru-cache';
import { VIEW_LIFETIME } from '../../../initializers/constants.js';
const lTags = loggerTagsFactory('views');
export class VideoViews {
    constructor() {
        this.viewsCache = new LRUCache({
            max: 10000,
            ttl: VIEW_LIFETIME.VIEW
        });
    }
    async addLocalView(options) {
        const { video, ip, watchTime } = options;
        logger.debug('Adding local view to video %s.', video.uuid, Object.assign({ watchTime }, lTags(video.uuid)));
        if (!await this.hasEnoughWatchTime(video, watchTime))
            return false;
        const viewExists = await this.doesVideoIPViewExist(ip, video.uuid);
        if (viewExists)
            return false;
        await this.setIPVideoView(ip, video.uuid);
        await this.addView(video);
        await sendView({ byActor: await getServerActor(), video, type: 'view', viewerIdentifier: buildUUID() });
        return true;
    }
    async addRemoteView(options) {
        const { video } = options;
        logger.debug('Adding remote view to video %s.', video.uuid, Object.assign({}, lTags(video.uuid)));
        await this.addView(video);
        return true;
    }
    async addView(video) {
        const promises = [];
        if (video.isOwned()) {
            promises.push(Redis.Instance.addLocalVideoView(video.id));
        }
        promises.push(Redis.Instance.addVideoViewStats(video.id));
        await Promise.all(promises);
    }
    async hasEnoughWatchTime(video, watchTime) {
        const { duration, isLive } = await getCachedVideoDuration(video.id);
        if (isLive || duration >= 30)
            return watchTime >= 30;
        return duration / watchTime < 2;
    }
    doesVideoIPViewExist(ip, videoUUID) {
        const key = Redis.Instance.generateIPViewKey(ip, videoUUID);
        const value = this.viewsCache.has(key);
        if (value === true)
            return Promise.resolve(true);
        return Redis.Instance.doesVideoIPViewExist(ip, videoUUID);
    }
    setIPVideoView(ip, videoUUID) {
        const key = Redis.Instance.generateIPViewKey(ip, videoUUID);
        this.viewsCache.set(key, true);
        return Redis.Instance.setIPVideoView(ip, videoUUID);
    }
}
//# sourceMappingURL=video-views.js.map